/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten;

import java.util.*;

/**
 * Ein Messquerschnitt.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Messquerschnitt implements Stoerfallindikator {

    private final String pid;
    private Linienreferenz liegtAuf;
    private Strassenteilsegment strassenteilsegment;
    private int anzahlFahrstreifen;

    public Messquerschnitt(String pid) {
        this.pid = Objects.requireNonNull(pid, "pid");
    }

    @Override
    public String getPid() {
        return pid;
    }

    public Optional<Linienreferenz> getLiegtAuf() {
        return Optional.ofNullable(liegtAuf);
    }

    public void setLiegtAuf(Linienreferenz liegtAuf) {
        this.liegtAuf = liegtAuf;
    }

    public Optional<Strassenteilsegment> getStrassenteilsegment() {
        if (strassenteilsegment == null) {
            bestimmeStrassenteilsegment();
        }
        return Optional.ofNullable(strassenteilsegment);
    }

    private void bestimmeStrassenteilsegment() {
        if (liegtAuf == null) {
            return;
        }

        Linie linie = liegtAuf.getLinie();
        if (linie instanceof Strassenteilsegment) {
            strassenteilsegment = (Strassenteilsegment) linie;
        } else if (linie instanceof Strassensegment) {
            Strassensegment s = (Strassensegment) linie;
            if (s.getTeilsegmente().isEmpty() || liegtAuf.getOffset().isNull()) {
                return;
            }

            double strecke = 0;
            for (int i = 0; i < s.getTeilsegmente().size(); i++) {
                Strassenteilsegment sts = s.getTeilsegmente().get(i);
                if (sts.getLaenge().equals(Laenge.UNDEFINIERT)) {
                    return;
                }

                strecke += sts.getLaenge().getWert();
                if (strecke >= liegtAuf.getOffset().getWert()) {
                    strassenteilsegment = sts;
                    return;
                }
            }
        }
    }

    @Override
    public Segmenttyp getTyp() {
        if (!getStrassenteilsegment().isPresent()) {
            return Segmenttyp.UNBEKANNT;
        }

        return getStrassenteilsegment().get().getTyp();
    }

    @Override
    public int getAnzahlFahrstreifen() {
        if (anzahlFahrstreifen > 0) {
            return anzahlFahrstreifen;
        } else if (getStrassenteilsegment().isPresent()) {
            return getStrassenteilsegment().get().getAnzahlFahrstreifen();
        } else {
            return 0;
        }
    }

    public void setAnzahlFahrstreifen(int anzahlFahrstreifen) {
        this.anzahlFahrstreifen = anzahlFahrstreifen;
    }

    @Override
    public String toString() {
        return "Messquerschnitt{" +
                "pid='" + pid + '\'' +
                '}';
    }

}
