/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten;

import java.util.*;

/**
 * Ein Straßenteilsegment.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Strassenteilsegment implements Stoerfallindikator, Linie {

    private final String pid;
    private Strassensegment gehoertZu;
    private Collection<Messquerschnitt> messquerschnitte = new ArrayList<>();
    private int anzahlFahrstreifen;
    private Laenge laenge = Laenge.UNDEFINIERT;

    public Strassenteilsegment(String pid) {
        this.pid = Objects.requireNonNull(pid, "pid");
    }

    @Override
    public String getPid() {
        return pid;
    }

    public Optional<Strassensegment> getGehoertZu() {
        return Optional.ofNullable(gehoertZu);
    }

    public void setGehoertZu(Strassensegment gehoertZu) {
        this.gehoertZu = gehoertZu;
    }

    public Collection<Messquerschnitt> getMessquerschnitte() {
        return messquerschnitte;
    }

    public void setMessquerschnitte(Collection<Messquerschnitt> messquerschnitte) {
        Objects.requireNonNull(messquerschnitte, "messquerschnitte");
        this.messquerschnitte = messquerschnitte;
    }

    @Override
    public Segmenttyp getTyp() {
        if (gehoertZu == null) {
            return Segmenttyp.UNBEKANNT;
        }

        return gehoertZu.getTyp();
    }

    @Override
    public int getAnzahlFahrstreifen() {
        return anzahlFahrstreifen;
    }

    public void setAnzahlFahrstreifen(int anzahlFahrstreifen) {
        this.anzahlFahrstreifen = anzahlFahrstreifen;
    }

    @Override
    public Laenge getLaenge() {
        return laenge;
    }

    public void setLaenge(Laenge laenge) {
        this.laenge = Objects.requireNonNull(laenge, "laenge");
    }

    @Override
    public String toString() {
        return "Strassenteilsegment{" +
                "pid='" + pid + '\'' +
                '}';
    }

}
