/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;
import de.bsvrz.sys.funclib.debug.*;

import java.text.*;
import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.Umfeldgroessen.*;

class GlaetteMesswertEmpfaenger extends Kommunikation {

    private final Event<Datensatz<Umfelddatenmessstelle, Collection<Messwert>>> messwertEmpfangen = new Event<>();

    private final Debug log = Debug.getLogger();
    private final ClientReceiverInterface receiver = new Receiver();
    private final ObjektFactory objektFactory;

    GlaetteMesswertEmpfaenger(ClientDavInterface verbindung, ObjektFactory objektFactory) {
        super(verbindung);
        this.objektFactory = objektFactory;
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        Collection<SystemObject> umfelddatenmessstellen = konvertiereObjekte(objekte, obj -> obj instanceof Umfelddatenmessstelle);
        DataDescription dd = datenbeschreibung("atg.ufdmsGlätte", "asp.prognose");
        getVerbindung().subscribeReceiver(receiver, umfelddatenmessstellen, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        log.config(MessageFormat.format("Als Empfänger von Glätte für {0} Umfelddatenmessstellen angemeldet.", umfelddatenmessstellen.size()));
    }

    private Datensatz<Umfelddatenmessstelle, Collection<Messwert>> konvertiereDatensatz(ResultData result) {
        Umfelddatenmessstelle objekt = konvertiereObjekt(result.getObject());
        Messwert wert = bestimmeMesswert(result.getData());
        Datensatz<Umfelddatenmessstelle, Collection<Messwert>> datensatz = new Datensatz<>(konvertiereZeitstempel(result.getDataTime()), objekt, Collections.singleton(wert));
        log.fine("Messwert für Glätte empfangen", datensatz);
        return datensatz;
    }

    private <T> T konvertiereObjekt(SystemObject objekt) {
        return (T) objektFactory.gibSystemobjekt(objekt);
    }

    private Messwert bestimmeMesswert(Data data) {
        int wert = data.getUnscaledValue("AktuellerZustand").intValue();
        if (wert < 0) {
            return new Messwert(GLAETTE);
        } else {
            return new Messwert(GLAETTE, wert);
        }
    }

    Event<Datensatz<Umfelddatenmessstelle, Collection<Messwert>>> messwertEmpfangen() {
        return messwertEmpfangen;
    }

    private class Receiver implements ClientReceiverInterface {

        @Override
        public void update(ResultData[] results) {
            for (ResultData r : results) {
                if (r.hasData()) {
                    messwertEmpfangen.send(konvertiereDatensatz(r));
                }
            }
        }

    }

}
