/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;
import de.bsvrz.sys.funclib.debug.*;

import java.text.*;
import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.Verkehrsgroessen.*;

class VerkehrsdatenKurzzeitMesswertEmpfaenger extends Kommunikation {

    private final Event<Datensatz<Messquerschnitt, Collection<Messwert>>> messwertEmpfangen = new Event<>();

    private final Debug log = Debug.getLogger();
    private final ClientReceiverInterface receiver = new Receiver();
    private final ObjektFactory objektFactory;

    VerkehrsdatenKurzzeitMesswertEmpfaenger(ClientDavInterface verbindung, ObjektFactory objektFactory) {
        super(verbindung);
        this.objektFactory = objektFactory;
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        Collection<SystemObject> messquerschnitte = konvertiereObjekte(objekte, obj -> obj instanceof Messquerschnitt);
        DataDescription dd = datenbeschreibung("atg.verkehrsDatenKurzZeitMq", "asp.analyse");
        getVerbindung().subscribeReceiver(receiver, messquerschnitte, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        log.config(MessageFormat.format("Als Empfänger von Kurzzeitdaten für {0} Messquerschnitte angemeldet.", messquerschnitte.size()));
    }

    protected Datensatz<Messquerschnitt, Collection<Messwert>> konvertiereDatensatz(ResultData result) {
        Messquerschnitt objekt = konvertiereObjekt(result.getObject());
        Collection<Messwert> messwerte = new ArrayList<>();
        messwerte.add(bestimmeMesswert(QKFZ, result.getData()));
        messwerte.add(bestimmeMesswert(VKFZ, result.getData()));
        messwerte.add(bestimmeMesswert(SKFZ, result.getData()));
        messwerte.add(bestimmeMesswert(ALKW, result.getData()));
        messwerte.add(bestimmeMesswert(KB, result.getData()));
        Datensatz<Messquerschnitt, Collection<Messwert>> datensatz = new Datensatz<>(konvertiereZeitstempel(result.getDataTime()), objekt, messwerte);
        log.fine("Messwert für Kurzzeitdaten empfangen", datensatz);
        return datensatz;
    }

    private <T> T konvertiereObjekt(SystemObject objekt) {
        return (T) objektFactory.gibSystemobjekt(objekt);
    }

    private Messwert bestimmeMesswert(String messgroesse, Data data) {
        Data.NumberValue wert = data.getItem(messgroesse).getScaledValue("Wert");
        if (wert.isNumber()) {
            return new Messwert(messgroesse, wert.intValue());
        } else {
            return new Messwert(messgroesse);
        }
    }

    Event<Datensatz<Messquerschnitt, Collection<Messwert>>> messwertEmpfangen() {
        return messwertEmpfangen;
    }

    private class Receiver implements ClientReceiverInterface {

        @Override
        public void update(ResultData[] results) {
            for (ResultData r : results) {
                if (r.hasData()) {
                    messwertEmpfangen.send(konvertiereDatensatz(r));
                }
            }
        }

    }

}
