/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.shared;

import java.util.*;
import java.util.concurrent.*;
import java.util.function.*;

/**
 * Veröffentlicht Ereignisse.
 *
 * @param <T> der Typ des Ereignisses.
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class Event<T> {

    private final List<Consumer<T>> handlers = new CopyOnWriteArrayList<>();

    public void addHandler(Consumer<T> handler) {
        Objects.requireNonNull(handler, "handler");
        handlers.add(handler);
    }

    public void removeHandler(Consumer<T> handler) {
        Objects.requireNonNull(handler, "handler");
        handlers.remove(handler);
    }

    public void send(T message) {
        handlers.forEach(handler -> {
            try {
                handler.accept(message);
            } catch (Exception e) {
                Thread.currentThread().getUncaughtExceptionHandler().uncaughtException(Thread.currentThread(), e);
            }
        });
    }

}
