/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.*;
import de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.*;
import de.bsvrz.sys.funclib.debug.*;
import de.bsvrz.sys.funclib.operatingMessage.*;

import java.text.*;
import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.Umfeldgroessen.*;
import static de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.Verkehrsgroessen.*;
import static de.bsvrz.iav.subsegana.subsegana.portals.Konstanten.*;

/**
 * Sendet Fuzzy-Sets an den Datenverteiler.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("ClassFanOutComplexity")
public class FuzzySetsSender extends Kommunikation {

    private static final String ASP_PID = "asp.parameterVorgabe";

    private final Debug log = Debug.getLogger();
    private final MessageSender messageSender = MessageSender.getInstance();
    private final ClientSenderInterface sender = new SenderOhneSendesteuerung();

    public FuzzySetsSender(ClientDavInterface verbindung) {
        super(verbindung);
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        meldeVerkehrsdatenAn(objekte);
        meldeUmfelddatenAn(objekte);
    }

    private void meldeVerkehrsdatenAn(Collection<Systemobjekt> objekte) {
        Collection<SystemObject> messquerschnitte = konvertiereObjekte(objekte, obj -> obj instanceof Messquerschnitt);
        anmelden(messquerschnitte, ATG_ANTEIL_LKW_FUZZYSETS);
        anmelden(messquerschnitte, ATG_BEMESSUNGSDICHTE_FUZZYSETS);
        anmelden(messquerschnitte, ATG_GESCHWINDIGKEIT_FUZZYSETS);
        anmelden(messquerschnitte, ATG_VERKEHRSSTAERKE_STUNDE_FUZZYSETS);
        anmelden(messquerschnitte, ATG_VERKEHRSUNRUHE_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Messquerschnitte angemeldet", messquerschnitte.size()));

        Collection<SystemObject> strassenteilsegmente = konvertiereObjekte(objekte, obj -> obj instanceof Strassenteilsegment);
        anmelden(strassenteilsegmente, ATG_ANTEIL_LKW_FUZZYSETS);
        anmelden(strassenteilsegmente, ATG_BEMESSUNGSDICHTE_FUZZYSETS);
        anmelden(strassenteilsegmente, ATG_GESCHWINDIGKEIT_FUZZYSETS);
        anmelden(strassenteilsegmente, ATG_VERKEHRSSTAERKE_STUNDE_FUZZYSETS);
        anmelden(strassenteilsegmente, ATG_VERKEHRSUNRUHE_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Straßenteilsegmente angemeldet.", strassenteilsegmente.size()));
    }

    private void meldeUmfelddatenAn(Collection<Systemobjekt> objekte) {
        Collection<SystemObject> umfelddatenmessstellen = konvertiereObjekte(objekte, obj -> obj instanceof Umfelddatenmessstelle);
        anmelden(umfelddatenmessstellen, ATG_GLAETTE_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Umfelddatenmessstellen angemeldet.", umfelddatenmessstellen.size()));

        Collection<SystemObject> helligkeitssensoren = konvertiereObjekte(objekte, obj -> obj instanceof Helligkeitssensor);
        anmelden(helligkeitssensoren, ATG_HELLIGKEIT_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Helligkeitssensoren angemeldet.", helligkeitssensoren.size()));

        Collection<SystemObject> niederschlagsintensitaetssensoren = konvertiereObjekte(objekte, obj -> obj instanceof Niederschlagsintensitaetssensor);
        anmelden(niederschlagsintensitaetssensoren, ATG_NIEDERSCHLAGSINTENSITAET_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Niederschlagsintensitätssensoren angemeldet.", niederschlagsintensitaetssensoren.size()));

        Collection<SystemObject> sichtweitesensoren = konvertiereObjekte(objekte, obj -> obj instanceof Sichtweitesensor);
        anmelden(sichtweitesensoren, ATG_SICHTWEITE_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Sichtweitesensoren angemeldet.", sichtweitesensoren.size()));

        Collection<SystemObject> windgeschwindigkeitssensoren = konvertiereObjekte(objekte, obj -> obj instanceof Windgeschwindigkeitssensor);
        anmelden(windgeschwindigkeitssensoren, ATG_WINDGESCHWINDIGKEIT_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Windgeschwindigkeitssensoren angemeldet.", windgeschwindigkeitssensoren.size()));

        Collection<SystemObject> windrichtungssensoren = konvertiereObjekte(objekte, obj -> obj instanceof Windrichtungssensor);
        anmelden(windrichtungssensoren, ATG_WINDRICHTUNG_FUZZYSETS);
        log.config(MessageFormat.format("Als Sender von Fuzzy-Sets für {0} Windrichtungssensoren angemeldet.", windrichtungssensoren.size()));
    }

    private void anmelden(Collection<SystemObject> objects, String attributgruppePid) {
        DataDescription dd = datenbeschreibung(attributgruppePid, ASP_PID);
        try {
            getVerbindung().subscribeSender(sender, objects, dd, SenderRole.sender());
        } catch (OneSubscriptionPerSendData e) {
            log.error("Sendeanmeldung fehlgeschlagen: " + e, Arrays.asList(objects, dd));
            throw new IllegalStateException("Sendeanmeldung fehlgeschlagen", e);
        }
    }

    public void sende(Collection<Datensatz<?, LinguistischeVariable>> datensaetze) {
        ResultData[] results = datensaetze.stream()
                .map(this::konvertiereDatensatz)
                .toArray(ResultData[]::new);
        try {
            getVerbindung().sendData(results);
            log.info(MessageFormat.format("{0} Fuzzy-Sets gesendet", datensaetze.size()));
        } catch (SendSubscriptionNotConfirmed e) {
            String message = "Parameter für Fuzzy-Sets konnten nicht gesendet werden: " + e;
            log.warning(message, e);
            messageSender.sendMessage(MessageType.SYSTEM_DOMAIN, MessageGrade.WARNING, message);
        }
    }

    private ResultData konvertiereDatensatz(Datensatz<?, LinguistischeVariable> datensatz) {
        SystemObject obj = getModell().getObject(datensatz.getSystemobjekt().getPid());
        AttributeGroup atg = bestimmeAttributgruppe(datensatz.getDatum());
        long time = konvertiereZeitstempel(datensatz.getZeitstempel());
        Data data = konvertiereDatum(datensatz.getDatum(), getVerbindung().createData(atg));
        return new ResultData(obj, datenbeschreibung(atg.getPid(), ASP_PID), time, data);
    }

    @SuppressWarnings("CyclomaticComplexity")
    private AttributeGroup bestimmeAttributgruppe(LinguistischeVariable fuzzySets) {
        String pid;
        switch (fuzzySets.getName()) {
            case QKFZ:
                pid = ATG_VERKEHRSSTAERKE_STUNDE_FUZZYSETS;
                break;
            case VKFZ:
                pid = ATG_GESCHWINDIGKEIT_FUZZYSETS;
                break;
            case SKFZ:
                pid = ATG_VERKEHRSUNRUHE_FUZZYSETS;
                break;
            case ALKW:
                pid = ATG_ANTEIL_LKW_FUZZYSETS;
                break;
            case KB:
                pid = ATG_BEMESSUNGSDICHTE_FUZZYSETS;
                break;
            case GLAETTE:
                pid = ATG_GLAETTE_FUZZYSETS;
                break;
            case HELLIGKEIT:
                pid = ATG_HELLIGKEIT_FUZZYSETS;
                break;
            case NIEDERSCHLAGSINTENSITAET:
                pid = ATG_NIEDERSCHLAGSINTENSITAET_FUZZYSETS;
                break;
            case SICHTWEITE:
                pid = ATG_SICHTWEITE_FUZZYSETS;
                break;
            case WINDGESCHWINDIGKEIT:
                pid = ATG_WINDGESCHWINDIGKEIT_FUZZYSETS;
                break;
            case WINDRICHTUNG:
                pid = ATG_WINDRICHTUNG_FUZZYSETS;
                break;
            default:
                throw new IllegalStateException("[unreachable code] Nicht behandelte linguistische Variable: " + fuzzySets.getName());
        }
        return getModell().getAttributeGroup(pid);
    }

    private static Data konvertiereDatum(LinguistischeVariable fuzzySets, Data data) {
        setValue(data.getItem("Minimum"), fuzzySets.getWertebereich().getMinimum());
        setValue(data.getItem("Maximum"), fuzzySets.getWertebereich().getMaximum());

        data.getItem("Urlasser").getReferenceValue("BenutzerReferenz").setSystemObject(null);
        data.getItem("Urlasser").getTextValue("Ursache").setText("Setzen von Standardparametern.");
        data.getItem("Urlasser").getTextValue("Veranlasser").setText("Straßensubsegmentanalyse");
        Data.Array array = data.getArray(fuzzySets.getName());
        array.setLength(fuzzySets.getTerme().size());
        for (int i = 0; i < array.getLength(); i++) {
            FuzzySet fuzzySet = fuzzySets.getFuzzySets().get(i);
            array.getItem(i).getTextValue("Name").setText(fuzzySet.getName());
            setValue(array.getItem(i).getItem("t1"), fuzzySet.getT1());
            setValue(array.getItem(i).getItem("t2"), fuzzySet.getT2());
            setValue(array.getItem(i).getItem("t3"), fuzzySet.getT3());
            setValue(array.getItem(i).getItem("t4"), fuzzySet.getT4());
        }
        return data;
    }

    private static void setValue(Data data, double value) {
        try {
            data.asScaledValue().set(value);
        } catch (IllegalArgumentException e) {
            data.asUnscaledValue().set(value);
        }
    }

}
